import os
import datetime
import uuid
from mambo import (Mambo, models, nav_menu, page_meta, init_app,
                         route, get_config, session, request, redirect,
                         url_for, get, post, flash, flash_data, flash_success, flash_error,
                         abort, recaptcha, storage, get_flash_data)

from flask_login import fresh_login_required
from mambo.exceptions import AppError
from mambo import utils
import json
from mambo.contrib.auth import (current_user, authenticated, is_authenticated,
                                not_authenticated)


def main(**kwargs):

    options = kwargs.get("options", {})
    nav_kwargs = kwargs.get("nav_menu", {})

    @nav_menu(title=nav_kwargs.pop("title", "Publisher Admin") or "Publisher Admin",
              visible=is_authenticated,
              css_id=nav_kwargs.pop("css_id", "publisher-admin-menu"),
              css_class=nav_kwargs.pop("css_class", "publisher-admin-menu"),
              order=nav_kwargs.pop("order", 20),
              **nav_kwargs)
    class PublisherAdmin(Mambo):
        base_route = kwargs.get("route") or "/publisher/"
        decorators = Mambo.decorators + [authenticated] + kwargs.get("decorators")

        @nav_menu("All Posts")
        def index(self):
            """
            List all posts
            """
            page_meta(title="All Posts")
            per_page = get_config("PAGINATION_PER_PAGE", 25)
            page = request.args.get("page", 1)
            id = request.args.get("id", None)
            slug = request.args.get("slug", None)
            status = request.args.get("status", "all")
            type_id = request.args.get("type_id", None)
            category_id = request.args.get("category_id", None)
            tag_id = request.args.get("tag_id", None)

            reorder_post = request.args.get("reorder_post")

            posts = models.PublisherPost.query()
            types = models.PublisherType.query().order_by("name asc")
            types_options = [(t.id, t.name) for t in types]

            selected_type = None
            if type_id:
                selected_type = models.PublisherType.get(type_id)

            if id:
                posts = posts.filter(models.PublisherPost.id == id)
            if slug:
                posts = posts.filter(models.PublisherPost.slug == slug)
            if type_id:
                type_id = int(type_id)
                posts = posts.filter(models.PublisherPost.type_id == type_id)
            if category_id:
                posts = posts.join(models.PublisherCategoryMap) \
                    .join(models.PublisherCategory) \
                    .filter(models.PublisherCategory.id == category_id)
            if tag_id:
                posts = posts.join(models.PublisherTagMap) \
                    .join(models.PublisherTag) \
                    .filter(models.PublisherTag.id == tag_id)
            if status == "publish":
                posts = posts.filter(models.PublisherPost.is_published == True)
            elif status == "draft":
                posts = posts.filter(models.PublisherPost.is_draft == True)
            elif status == "revision":
                posts = posts.filter(models.PublisherPost.is_revision == True)

            posts = posts.order_by(models.PublisherPost.id.desc())
            posts = posts.paginate(page=page, per_page=per_page)

            return dict(posts=posts,
                        types=types,
                        types_options=types_options,
                        search=False,
                        query_vars={
                            "id": id,
                            "slug": slug,
                            "type_id": type_id,
                            "status": status
                        },
                        selected_type=selected_type)

        @nav_menu("Post Preview", endpoint="PublisherAdmin:preview", visible=False)
        def preview(self, id):
            """
            Read Post
            """
            post = models.PublisherPost.get(id)
            if not post:
                abort(404, "Post doesn't exist")

            page_meta(title="Read: %s " % post.title)

            return dict(post=post)

        @nav_menu("Edit Post", endpoint="PublisherAdmin:edit", visible=False)
        @nav_menu("New Post", endpoint="PublisherAdmin:new", order=2)
        @get("/new/", defaults={"id": None}, endpoint="PublisherAdmin:new")
        @get("/edit/<id>/", endpoint="PublisherAdmin:edit")
        def edit(self, id):
            """
            Create / Edit Post
            """
            page_meta(title="Edit Post")

            types = [(t.id, t.name) for t in
                     models.PublisherType.query().order_by(models.PublisherType.name.asc())]
            categories = [(c.id, c.name) for c in
                          models.PublisherCategory.query().order_by(
                              models.PublisherCategory.name.asc())]
            checked_cats = []

            type_id = request.args.get("type_id", None)

            # data to pass to view
            post = {
                "id": 0,
                "title": "",
                "content": "",
                "slug": "",
                "is_public": True,
                "is_sticky": False,
                "is_featured": False,
                "type_id": 0 if not type_id else int(type_id),
                "options": {}
            }

            # saved in session
            flashed_data = get_flash_data()
            if request.args.get("error") and flashed_data:
                post = flashed_data
                checked_cats = post["post_categories"]

            elif id:
                post = models.PublisherPost.get(id)
                if not post or post.is_revision:
                    abort(404, "Post doesn't exist")
                checked_cats = [c.id for c in post.categories]

            images = models.PublisherAssets.query() \
                .filter(models.PublisherAssets.type == "IMAGE") \
                .order_by(models.PublisherAssets.name.asc())

            images_list = [{"id": img.id, "url": img.object_url} for img in images]
            return dict(post=post,
                        types=types,
                        categories=categories,
                        checked_categories=checked_cats,
                        images_list=images_list)

        @post()
        def save_post(self):
            id = request.form.get("id")
            title = request.form.get("title")
            slug = request.form.get("slug")
            content = request.form.get("content")
            description = request.form.get("description")
            summary = request.form.get("summary")
            keywords = request.form.get("keywords")
            type_id = request.form.get("type_id")
            post_categories = request.form.getlist("post_categories")
            published_date = request.form.get("published_date")
            status = request.form.get("status", "draft")
            is_published = True if status == "publish" else False
            is_draft = True if status == "draft" else False
            is_public = True if request.form.get("is_public") == "y" else False
            is_sticky = True if request.form.get("is_sticky") == "y" else False
            is_featured = True if request.form.get("is_featured") == "y" else False
            featured_image = request.form.get("featured_image")
            featured_embed = request.form.get("featured_embed")
            featured_media_top = request.form.get("featured_media_top", "")
            social_options = request.form.getlist("social_options")
            tags = list(set(request.form.get("tags", "").split(",")))

            now_dt = datetime.datetime.now()
            data = {
                "title": title,
                "content": content,
                "description": description,
                "summary": summary,
                "keywords": keywords,
                "featured_image": featured_image,
                "featured_embed": featured_embed,
                "featured_media_top": featured_media_top,
                "type_id": type_id,
                "is_sticky": is_sticky,
                "is_featured": is_featured,
                "is_public": is_public
            }

            if status in ["draft", "publish"] and (not title or not type_id):
                if not title:
                    flash_error("Post Title is missing ")
                if not type_id:
                    flash_error("Post type is missing")

                data.update({
                    "published_date": published_date,
                    "post_categories": post_categories,
                    "options": {"social_options": social_options},
                })
                flash_data(data)

                if id:
                    url = url_for(self.edit, id=id, error=1)
                else:
                    url = url_for(self.new, error=1)
                return redirect(url)

            published_date = datetime.datetime.strptime(published_date,
                                                        "%Y-%m-%d %H:%M:%S") \
                if published_date else now_dt

            if id and status in ["delete", "revision"]:
                post = models.PublisherPost.get(id)
                if not post:
                    abort(404, "Post '%s' doesn't exist" % id)

                if status == "delete":
                    post.delete()
                    flash_success("Post deleted successfully!")
                    return redirect(self.index)

                elif status == "revision":
                    data.update({
                        "parent_id": id,
                        "is_revision": True,
                        "is_draft": False,
                        "is_published": False,
                        "is_public": False
                    })
                    post = models.PublisherPost.create(**data)
                    return jsonify({"revision_id": post.id})

            elif status in ["draft", "publish"]:
                data.update({
                    "is_published": is_published,
                    "is_draft": is_draft,
                    "is_revision": False,
                    "is_public": is_public
                })

                if id:
                    post = models.PublisherPost.get(id)
                    if not post:
                        abort(404, "Post '%s' doesn't exist" % id)
                    elif post.is_revision:
                        abort(403, "Can't access this post")
                    else:
                        if is_sticky and not post.is_sticky:
                            data["sticky_at"] = now_dt
                        if is_featured and not post.is_featured:
                            data["featured_at"] = now_dt
                        post.update(**data)
                else:
                    if is_published:
                        data["published_at"] = published_date
                    if is_sticky:
                        data["sticky_at"] = now_dt
                    if is_featured:
                        data["featured_at"] = now_dt
                    post = models.PublisherPost.create(**data)

                # prepare tags
                _tags = []
                for tag in tags:
                    tag = tag.strip().lower()
                    _tag = models.PublisherTag.get_by_slug(name=tag)
                    if tag and not _tag:
                        _tag = models.PublisherTag.new(name=tag)
                    if _tag:
                        _tags.append(_tag.id)
                post.update_tags(_tags)

                post.set_slug(slug or title)
                post.update_categories(map(int, post_categories))
                post.set_options("social", social_options)

                if post.is_published and not post.published_at:
                    post.update(published_at=published_date)

                flash_success("Post saved successfully!")

                return redirect(self.edit, id=post.id)

            else:
                abort(400, "Invalid post status")

        @nav_menu("Categories", order=4)
        @get()
        @post()
        def categories(self):
            page_meta(title="Post Categories")
            if request.method == "POST":
                id = request.form.get("id", None)
                action = request.form.get("action")
                name = request.form.get("name")
                slug = request.form.get("slug", None)
                ajax = request.form.get("ajax", False)
                try:
                    if not id:
                        cat = models.PublisherCategory.new(name=name, slug=slug)
                        if ajax:
                            return jsonify({
                                "id": cat.id,
                                "name": cat.name,
                                "slug": cat.slug,
                                "status": "OK"
                            })
                        flash_success("New category '%s' added" % name)
                    else:
                        post_cat = models.PublisherCategory.get(id)
                        if post_cat:
                            if action == "delete":
                                post_cat.delete()
                                flash_success("Category '%s' deleted successfully!" % post_cat.name)
                            else:
                                post_cat.update(name=name, slug=slug)
                                flash_success("Category '%s' updated successfully!" % post_cat.name)
                except Exception as ex:
                    if ajax:
                        return jsonify({
                            "error": True,
                            "error_message": ex.message
                        })

                    flash_error("Error: %s" % ex.message)
                return redirect(self.categories)

            else:
                cats = models.PublisherCategory.query().order_by(
                    models.PublisherCategory.name.asc())
                return dict(categories=cats)

        @nav_menu("Tags", order=5)
        @get()
        @post()
        def tags(self):
            page_meta(title="Post Tags")
            if request.method == "POST":
                id = request.form.get("id", None)
                action = request.form.get("action")
                name = request.form.get("name")
                slug = request.form.get("slug", None)
                ajax = request.form.get("ajax", False)
                try:
                    if not id:
                        tag = models.PublisherTag.new(name=name, slug=slug)
                        if ajax:
                            return jsonify({
                                "id": tag.id,
                                "name": tag.name,
                                "slug": tag.slug,
                                "status": "OK"
                            })
                        flash_success("New Tag '%s' added" % name)
                    else:
                        post_tag = models.PublisherTag.get(id)
                        if post_tag:
                            if action == "delete":
                                post_tag.delete()
                                flash_success("Tag '%s' deleted successfully!" % post_tag.name)
                            else:
                                post_tag.update(name=name, slug=slug)
                                flash_success("Tag '%s' updated successfully!" % post_tag.name)
                except Exception as ex:
                    if ajax:
                        return jsonify({
                            "error": True,
                            "error_message": ex.message
                        })

                    flash_error("Error: %s" % ex.message)
                return redirect(self.tags)

            else:
                tags = models.PublisherTag.query().order_by(models.PublisherTag.name.asc())
                return dict(tags=tags)

        @nav_menu("Types", order=6)
        @get()
        @post()
        def types(self):
            page_meta(title="Post Types")
            if request.method == "POST":
                try:
                    id = request.form.get("id", None)
                    action = request.form.get("action")
                    name = request.form.get("name")
                    slug = request.form.get("slug", None)
                    if not id:
                        models.PublisherType.new(name=name, slug=slug)
                        flash_success("New type '%s' added" % name)
                    else:
                        post_type = models.PublisherType.get(id)
                        if post_type:
                            if action == "delete":
                                post_type.delete()
                                flash_success("Type '%s' deleted successfully!" % post_type.name)
                            else:
                                post_type.update(name=name, slug=slug)
                                flash_success("Type '%s' updated successfully!" % post_type.name)
                except Exception as ex:
                    flash_error("Error: %s" % ex.message)
                return redirect(self.types)
            else:
                types = models.PublisherType.query().order_by(models.PublisherType.name.asc())
                return dict(types=types)

        @nav_menu("Images", order=3)
        @get()
        @post()
        def images(self):
            page_meta(title="Images")
            if request.method == "POST":
                id = request.form.get("id", None)
                action = request.form.get("action")
                description = request.form.get("description")
                if id:
                    image = models.PublisherAssets.get(id)
                    if image:
                        if action == "delete":
                            image.delete()
                            obj = storage.get(image.name)
                            if obj:
                                obj.delete()
                            flash_success("Image deleted successfully!")
                        else:
                            image.update(description=description)
                            flash_success("Image updated successfully!")
                else:
                    abort(404, "No image ID provided")
                return redirect(self.images)

            else:
                page = request.args.get("page", 1)
                per_page = get_config("PAGINATION_PER_PAGE", 25)
                images = models.PublisherAssets.query() \
                    .filter(models.PublisherAssets.type == "IMAGE") \
                    .order_by(models.PublisherAssets.name.asc())
                images = images.paginate(page=page, per_page=per_page)
                return dict(images=images)

        @post()
        def upload_image(self):
            """
            Placeholder for markdown
            """
            try:
                ajax = request.form.get("ajax", False)
                allowed_extensions = ["gif", "png", "jpg", "jpeg"]

                if request.files.get("file"):
                    _file = request.files.get('file')
                    obj = storage.upload(_file,
                                         prefix="uploads/publisher/",
                                         allowed_extensions=allowed_extensions,
                                         public=True)

                    if obj:
                        description = os.path.basename(obj.name)
                        description = description.replace(".%s" % obj.extension, "")
                        description = description.split("__")[0]
                        upload_object = models.PublisherAssets.create(name=obj.name,
                                                                      provider=obj.provider_name,
                                                                      container=obj.container.name,
                                                                      extension=obj.extension,
                                                                      type=obj.type,
                                                                      object_path=obj.path,
                                                                      object_url=obj.url,
                                                                      size=obj.size,
                                                                      description=description)
                        if ajax:
                            return jsonify({
                                "id": upload_object.id,
                                "url": upload_object.object_url
                            })
                        else:
                            flash_success("Image '%s' uploaded successfully!" % upload_object.name)
                else:
                    flash_error("Error: Upload object file is invalid or doesn't exist")
            except Exception as e:
                flash_error("Error: %s" % e.message)
            return redirect(self.images)

    return PublisherAdmin



import datetime
import uuid
from mambo import (Mambo, models, nav_menu, page_meta, init_app,
                         route, get_config, session, request, redirect,
                         url_for, get, post, flash_success, flash_error,
                         abort, recaptcha, storage)

from flask_login import fresh_login_required
from mambo.exceptions import AppError
from mambo import utils
from . import (current_user, authenticated, logout_user,
               is_authenticated, not_authenticated, require_login_allowed,
               require_signup_allowed, require_social_login_allowed,
               send_mail_password_reset, send_mail_verification_email,
               send_mail_signup_welcome, session_set_require_password_change)
import arrow


def main(**kwargs):

    options = kwargs.get("options", {})
    nav_title_partial = "AuthAccount/nav_title_partial.html"
    nav_kwargs = kwargs.get("nav_menu", {})
    verify_email = options.get("verify_email") or False

    @nav_menu(title=nav_kwargs.pop("title", "My Account") or "My Account",
              visible=is_authenticated,
              css_id=nav_kwargs.pop("css_id", "auth-account-menu"),
              css_class=nav_kwargs.pop("css_class", "auth-account-menu"),
              align_right=nav_kwargs.pop("align_right", True),
              title_partial=nav_kwargs.pop("title_partial", nav_title_partial),
              **nav_kwargs)
    class AuthAccount(Mambo):
        base_route = kwargs.get("route") or "/"
        decorators = Mambo.decorators + [authenticated] + kwargs.get("decorators")

        def _confirm_password(self):
            user_login = current_user.user_login("email")
            password = request.form.get("confirm-password")
            if not user_login.password_matched(password):
                raise AppError("Invalid password")
            return True

        @nav_menu("Logout", endpoint="AuthLogin:logout", order=100)
        def _(self):
            pass

        @nav_menu("Account Info", order=1)
        def account_info(self):
            page_meta(title="Account Info")

        @nav_menu("Edit Account Info", visible=False)
        @get()
        @post()
        def edit_account_info(self):
            page_meta(title="Edit Account Info")

            if request.method == "POST":
                try:
                    data = {
                        "name": request.form.get("name"),
                        "first_name": request.form.get("first_name"),
                        "last_name": request.form.get("last_name"),
                        "address": request.form.get("address"),
                        "address_city": request.form.get("address_city"),
                        "address_state": request.form.get("address_state"),
                        "address_country": request.form.get("address_country"),
                        "address_zip_code": request.form.get("address_zip_code"),
                        "telephone": request.form.get("telephone"),
                    }
                    dob = request.form.get("dob")
                    if dob:
                        data["date_of_birth"] = arrow.get(dob, models.AuthUser.DOB_FORMAT).datetime
                    current_user.update(**data)

                    flash_success("Account info updated successfully!")

                except AppError as ae:
                    flash_error(ae.message)
                return redirect(self.account_info)

        @nav_menu("Change Login", order=2)
        @get()
        @post()
        def change_email(self):
            page_meta(title="Change Login Email")

            user_login = current_user.user_login("email")
            if not user_login:
                abort(400, "Require and Email account login to continue")

            if request.method == "POST":
                try:
                    self._confirm_password()

                    email = request.form.get("email")
                    if utils.is_email_valid(email):
                        user_login.change_email(email)
                        flash_success("Login Email changed successfully!")
                        return redirect(self.account_info)
                    else:
                        raise AppError("Invalid email")
                except AppError as ex:
                    flash_error("Error: %s" % ex)
                    return redirect(self.change_email)

            return {
                "user_login": user_login
            }

        @nav_menu("Change Password", order=3)
        @get()
        @post()
        def change_password(self):
            page_meta(title="Change Password")

            user_login = current_user.user_login("email")
            if not user_login:
                abort(400, "Require and Email account login to continue")

            if request.method == "POST":
                try:
                    self._confirm_password()
                    password = request.form.get("password")
                    password2 = request.form.get("password2")
                    if password != password2:
                        raise AppError("Passwords don't match")
                    elif utils.is_password_valid(password):
                        user_login.change_password(password.strip())
                        session_set_require_password_change(False)
                        flash_success("Password updated successfully!")
                        return redirect(self.account_info)
                    else:
                        raise AppError("Invalid password")
                except AppError as ex:
                    flash_error("Error: %s" % ex)
                    return redirect(self.change_password)

        @nav_menu("Change Profile Image", order=4)
        @get()
        @post()
        def change_profile_image(self):
            page_meta(title="Change Profile Image")

            if request.method == "POST":
                file = request.files.get("file")
                if file:
                    name = "%s" % uuid.uuid4().hex
                    extensions = ["jpg", "jpeg", "png", "gif"]
                    my_photo = storage.upload(file,
                                              name=name,
                                              prefix="profile-images/",
                                              allowed_extensions=extensions)
                    if my_photo:
                        url = my_photo.url
                        current_user.update(profile_image_url=url)
                        flash_success("Profile Picture updated successfully!")
                return redirect(self.account_info)

        @nav_menu("Setup Login", visible=False)
        @get()
        @post()
        def setup_login(self):
            user_login = current_user.user_login("email")
            if user_login:
                return redirect(self.account_info)

            if request.method == "POST":
                try:
                    email = request.form.get("email")
                    password = request.form.get("password")
                    password2 = request.form.get("password2")

                    if not password.strip() or password.strip() != password2.strip():
                        raise AppError("Passwords don't match")
                    else:
                        new_login = models.AuthUserLogin.new(login_type="email",
                                                         user_id=current_user.id,
                                                         email=email,
                                                         password=password.strip())
                        if verify_email:
                            send_mail_signup_welcome(new_login)
                            flash_success("A welcome email containing a confirmation link has been sent to %s" % email)
                            return redirect(self.account_info)
                except AppError as ex:
                    flash_error(ex.message)
                return redirect(self.setup_login)

    return AuthAccount







import datetime
from mambo import (Mambo, models, views, nav_menu, page_meta, init_app,
                         route, get_config, session, request, redirect,
                         url_for, get, post, flash_success, flash_error, flash, abort, recaptcha)
from flask_login import (LoginManager, login_user, current_user, fresh_login_required)
from . import (logout_user, is_authenticated, not_authenticated,
                send_mail_password_reset, send_mail_verification_email, send_mail_signup_welcome,
               require_login_allowed, require_signup_allowed, require_social_login_allowed,
               session_set_require_password_change, _get_app_options,

                login_with_email, signup_with_email, session_login
               )
from mambo.exceptions import AppError
from mambo import utils
import logging
import exceptions


def main(**kwargs):
    """
    This plugin allow user to login to application

        - options:
            - login_view
            - logout_view
            - verify_email

    """

    options = kwargs.get("options", {})

    login_view = options.get("login_view") or "Index:index"
    logout_view = options.get("logout_view") or "Index:login"
    verify_email = options.get("verify_email") or False

    login_manager = LoginManager()
    login_manager.login_view = "AuthLogin:login"
    login_manager.login_message_category = "error"
    init_app(login_manager.init_app)

    @login_manager.user_loader
    def load_user(userid):
        return models.AuthUser.get(userid)

    def nav_menu_visible_signup():
        if not _get_app_options().get("allow_signup", False):
            return False
        return not_authenticated()

    def nav_menu_visible_login():
        if not _get_app_options().get("allow_login", False):
            return False
        return not_authenticated()

    navigation = kwargs.get("nav_menu", {})
    navigation.setdefault("title", None)
    navigation.setdefault("visible", True)
    navigation.setdefault("order", 100)
    navigation["visible"] = not_authenticated

    Mambo.g(__AUTH_ENABLED__=True)

    @nav_menu(**navigation)
    class AuthLogin(Mambo):
        base_route = kwargs.get("route") or "/"

        @nav_menu("Login", visible=nav_menu_visible_login)
        @get()
        @post()
        @require_login_allowed
        @logout_user
        def login(self):
            page_meta(title="Login")

            if request.method == "POST":
                if request.form.get("method") == "email":
                    return self._post_login_email()
                elif request.form.get("method") == "social":
                    return self._post_login_social()
                else:
                    abort(400, "Invalid login method")

            return {
                "email": request.args.get("email"),
                "login_url_next": request.args.get("next", ""),
                "allow_signup": options.get("allow_signup"),
                "show_verification_message": True if request.args.get("v") == "1" else False
            }

        def _post_login_email(self):
            email = request.form.get("email").strip()
            password = request.form.get("password").strip()

            try:
                if not email or not password:
                    flash_error("Email or Password is empty")
                    return redirect(self.login, next=request.form.get("next"))

                userl = login_with_email(email=email,
                                         password=password,
                                         verify_email=verify_email)
                if userl:
                    if userl.require_password_change is True:
                        flash("Password change is required", "info")
                        session_set_require_password_change(True)
                        return redirect(views.AuthAccount.change_password)
                    return redirect(request.form.get("next") or login_view)
                else:
                    flash_error("Email or Password is invalid")
                    return redirect(self.login, next=request.form.get("next"))

            except exceptions.VerifyEmailError as ve:
                return redirect(self.login, email=email, v="1")

            except Exception as e:
                logging.exception(e)
                flash_error("Unable to login")
                return redirect(self.login, next=request.form.get("next"))

        def _post_login_social(self):
            return redirect(self.login)

        @nav_menu("Logout", visible=False)
        @get()
        @logout_user
        def logout(self):
            session_set_require_password_change(False)
            return redirect(logout_view or self.login)

        @nav_menu("Lost Password", visible=False)
        @get()
        @post()
        @require_login_allowed
        @logout_user
        def lost_password(self):
            page_meta(title="Lost Password")

            if request.method == "POST":
                email = request.form.get("email")
                user_l = models.AuthUserLogin.get_by_email(email)
                if user_l:
                    send_mail_password_reset(user_l)
                    flash_success("A new password has been sent to '%s'" % email)
                    return redirect(self.login)
                else:
                    flash_error("Invalid email address")
                    return redirect(self.lost_password)

        @nav_menu("Signup", visible=nav_menu_visible_signup)
        @get()
        @post()
        @require_login_allowed
        @require_signup_allowed
        @logout_user
        def signup(self):
            """
            For Email Signup
            :return:
            """
            page_meta(title="Signup")
            if request.method == "POST":
                if request.form.get("method") == "email":
                    return self._post_signup_email()
                elif request.form.get("method") == "social":
                    return self._post_signup_social()
                else:
                    abort(400, "Invalid signup method")

            return dict(login_url_next=request.args.get("next", ""),)

        def _post_signup_email(self):
            if not recaptcha.verify():
                flash_error("Invalid Security code")
                return redirect(self.signup, next=request.form.get("next"))
            try:
                name = request.form.get("name")
                email = request.form.get("email")
                password = request.form.get("password")
                password2 = request.form.get("password2")

                if not name:
                    raise AppError("Name is required")
                elif not password.strip() or password.strip() != password2.strip():
                    raise AppError("Passwords don't match")
                else:
                    new_login = signup_with_email(email=email,
                                                  password=password,
                                                  name=name)
                    if verify_email:
                        send_mail_signup_welcome(new_login)
                        flash_success("A welcome email containing a confirmation link has been sent to %s" % email)
                    else:
                        session_login(new_login)
                        print(request.form.get("next") or login_view)
                    return redirect(request.form.get("next") or login_view)
            except AppError as ex:
                flash_error(ex.message)
            return redirect(self.signup, next=request.form.get("next"))

        @nav_menu("Reset Password", visible=False)
        @get("/reset-password/<token>/")
        @post("/reset-password/<token>/")
        @require_login_allowed
        @logout_user
        def reset_password(self, token):
            page_meta(title="Reset Password")
            user_login = models.AuthUserLogin.get_by_temp_login(token)
            if user_login:
                if request.method == "POST":
                    try:
                        password = request.form.get("password", "").strip()
                        password2 = request.form.get("password2", "").strip()
                        if not password:
                            raise AppError("Password is missing")
                        elif password != password2:
                            raise AppError("Password don't match")

                        user_login.change_password(password)
                        user_login.clear_temp_login()
                        user_login.clear_email_verified_token()

                        session_set_require_password_change(False)
                        flash_success("Password updated successfully!")
                        return redirect(login_view)
                    except AppError as ex:
                        flash_error("Error: %s" % ex.message)
                        return redirect(self.reset_password, token=token)
                return {"token": token}
            return redirect(self.login)

        @nav_menu("Confirm Email", visible=False)
        @get()
        @post()
        @require_login_allowed
        @logout_user
        def confirm_email(self):
            if not verify_email:
                return redirect(self.login)

            if request.method == "POST":
                email = request.form.get("email")
                if email and utils.is_email_valid(email):
                    userl = models.AuthUserLogin.get_by_email(email)
                    if userl:
                        if not userl.email_verified:
                            send_mail_verification_email(userl)
                            flash_success("A verification email has been sent to %s" % email)
                        return redirect(self.login, email=email)
                flash_error("Invalid account")
                return redirect(self.confirm_email, email=email)

            page_meta(title="Confirm Email")
            return {
                "email": request.args.get("email"),
            }

        @require_login_allowed
        @logout_user
        def verify_email(self, token):
            user_login = models.AuthUserLogin.get_by_email_verified_token(token)
            if user_login:
                user_login.clear_email_verified_token()
                flash_success("Account verified. You can now login")
                return redirect(self.login, email=user_login.email)
            return redirect(self.login)
            
    return AuthLogin



"""
Mambo

Views

"""
from mambo import (Mambo, page_meta, get_config, flash_success, flash_error,
                   abort, request, url_for, redirect, models,
                   # decorators
                   nav_menu, get, post, render_json)

# ------------------------------------------------------------------------------


class Index(Mambo):

    @nav_menu("Home", order=1)
    def index(self):
        page_meta(title="Hello View!")
        return {}

    @get("/hello-json/")
    @render_json
    def hello(self):
        return {"Hello": "World"}





# Mambo

import os

# ------------------------------------------------------------------------------
# A convenient utility to access data path from your application and config files

# The application directory
# /application
APP_DIR = os.path.dirname(__file__)

# Data directory
# application/_data
DATA_DIR = os.path.join(APP_DIR, "_data")


def get_app_data_path(path):
    """
    get the path stored in the 'application/_data' directory
    :param path: string
    :return: string
    """
    return os.path.join(DATA_DIR, path)

# ------------------------------------------------------------------------------
#
# AVAILABLE APPS
# Dict of available apps to be used in your app config
# To activate apps, just place each one of them in the
# INSTALLED_APPS config. ie
#
# INSTALLED_APPS = [
#     AVAILABLE_APPS["ERROR_PAGE"],
#     AVAILABLE_APPS["AUTH"],
#     ...
# ]
#
# The order is important. If an app depends on another app to work
# that must be placed before the calling app.
#
# Multiple app config of the same app can also be set to be used by different
# app. ie:
#
# AVAILABLE_APPS = {
#     "AUTH_ADMIN": {...},
#     "AUTH_WWW": {...}
# }
#
#
# INSTALLED_APPS = [
#     AVAILABLE_APPS["AUTH_WWW"],
#     ...
# ]
#
#

AVAILABLE_APPS = {
    # Error Page. Create a friendly page when an error occurs
    "ERROR_PAGE": "mambo.contrib.error_page",

    # Maintenance page. When uncommented, the whole site will show a maintenance page
    "MAINTENANCE_PAGE": "mambo.contrib.maintenance_page",

    "AUTH":
    # AUTH: Authentication system to signup, login, manage users,
    # give access etc
    # Require to run `mambo syncdb` to setup the db
    # Also, run once `mambo auth:create-super-admin email@xyz.com` to
    # create the super admin
        {
            "app": "mambo.contrib.auth",
            "modules": {
                "login": {
                    "route": "/account/",
                    "nav_menu": {
                        "title": "Signin",
                        "align_right": True
                    }
                },
                "account": {
                    "route": "/account/",
                },
                "admin": {
                    "route": "/admin/users/"
                }
            },
            "options": {
                # for login and logout view
                "login_view": "Index:index",
                "logout_view": "Index:index",

                # permission
                "allow_signup": True,
                "allow_login": True,
                "allow_social_login": False,

                # Verification
                "verify_email": False,
                "verify_email_token_ttl": 60 * 24,
                "verify_email_template": "verify-email.txt",
                "verify_signup_email_template": "verify-signup-email.txt",

                # reset password
                "reset_password_method": "token",  # token or password
                "reset_password_token_ttl": 60,  # in minutes
                "reset_password_email_template": "reset-password.txt",

                # # When a user is required to change password, list endpoints
                # # that could still be accessed without requiring password
                # # change page
                # "require_password_change_exclude_endpoints": []
            }
        },
    "CONTACT_PAGE":
    # CONTACT PAGE: Creates a page for users to contact admin.
    # MAIL_* config must be setup
        {
            "app": "mambo.contrib.contact_page",
            "route": "/contact/",
            "nav_menu": {
                "title": "Contact",
                "visible": True,
                "order": 100
            },
            "options": {
                "title": "Get in touch!",
                "return_to": "Index:index",
                "recipients": "",
                "template": "contact-us.txt",
                "success_message": "Thank you soooooo much for sending this message. "
                                   "We'll contact you within the next 72 hours"
            }
        },
    "PUBLISHER":
    #
    # PUBLISHER: A pseudo CMS to add content to the site.
    # Require to run `mambo syncdb` to setup the db
        {
            "app": "mambo.contrib.publisher",
            "modules": {
                "admin": {
                    "route": "/admin/publisher"
                }
            }
        },

    "ADMIN":
    # ADMIN: Creates and admin interface for the application.
    # Use when creating admin site
        {
            "app": "mambo.contrib.admin",
            # For additional roles in the whole admin page, uncomment below
            #"decorators": ["mambo.contrib.auth.accepts_manager_roles"],
            "options": {
                "brand": "The Admin",
                "theme": "yeti"  # Recommended: yeti, superhero, amelia
            }

        },

    "APP_DATA": {
        "app": "mambo.contrib.app_data",
        "modules": {
            "admin": {}
        }
    }
}




from mambo import emit_signal, models
from blinker import Namespace

ns = Namespace()


@emit_signal(namespace=ns)
def on_signup(cb):
    """
    Emit a signal when signing up
    :param cb: callback function to execute
    :return: AuthUserLogin
    """
    return cb()


@emit_signal(namespace=ns)
def on_login(cb):
    """
    Emit signal when logged in
    :param cb: callback function to execute
    :return: AuthUserLogin
    """
    return cb()


@emit_signal(namespace=ns)
def on_logout(cb):
    """
    :param cb: callback function to execute
    :return: Current user
    """
    return cb()


@emit_signal(namespace=ns)
def on_account_info_change(cb):
    """
    :param cb: callback function to execute
    :return:
    """
    return cb()

@emit_signal(namespace=ns)
def on_login_change(cb):
    """
    :param cb: callback function to execute
    :return:
    """
    return cb()


@emit_signal(namespace=ns)
def on_password_change(cb):
    """
    :param cb: callback function to execute
    :return:
    """
    return cb()

@emit_signal(namespace=ns)
def on_verify_email(cb):
    """
    :param cb: callback function to execute
    :return: AuthUserLogin
    """
    return cb()

@emit_signal(namespace=ns)
def on_profile_image_change(cb):
    """
    :param cb: callback function to execute
    :return:
    """
    return cb()

@emit_signal(namespace=ns)
def on_reset_password(cb):
    """
    :param cb: callback function to execute
    :return: AuthUserLogin
    """
    return cb()


@emit_signal(namespace=ns)
def on_confirm_email(cb):
    """
    :param cb: callback function to execute
    :return: AuthUserLogin
    """
    return cb()





"""
________________________________________________________________________________

 /$$      /$$  /$$$$$$  /$$      /$$ /$$$$$$$   /$$$$$$
| $$$    /$$$ /$$__  $$| $$$    /$$$| $$__  $$ /$$__  $$
| $$$$  /$$$$| $$  \ $$| $$$$  /$$$$| $$  \ $$| $$  \ $$
| $$ $$/$$ $$| $$$$$$$$| $$ $$/$$ $$| $$$$$$$ | $$  | $$
| $$  $$$| $$| $$__  $$| $$  $$$| $$| $$__  $$| $$  | $$
| $$\  $ | $$| $$  | $$| $$\  $ | $$| $$  \ $$| $$  | $$
| $$ \/  | $$| $$  | $$| $$ \/  | $$| $$$$$$$/|  $$$$$$/
|__/     |__/|__/  |__/|__/     |__/|_______/  \______/


https://github.com/mardix/mambo

________________________________________________________________________________
"""

import os
import re
import sys
import traceback
import logging
import importlib
import pkg_resources
import click
import yaml
import functools
import sh
from werkzeug import import_string
from .__about__ import *

CWD = os.getcwd()
SKELETON_DIR = "skel"
APPLICATION_DIR = "%s/application" % CWD
APPLICATION_DATA_DIR = "%s/_data" % APPLICATION_DIR
APPLICATION_BABEL_DIR = "%s/babel" % APPLICATION_DATA_DIR

class MamboCLI(object):
    """
    For command line classes in which __init__ contains all the functions to use

    example

    class MyCLI(MamboCLI):
        def __init__(self):

            @cli.command()
            def hello():
                click.echo("Hello world")

            @cli.command()
            @click.argument("name")
            def say_my_name(name):
                click.echo("My name is %s" % name)
    """
    pass


def get_project_dir_path(project_name):
    return "%s/%s" % (APPLICATION_DIR, project_name)


def copy_resource(src, dest):
    """
    To copy package data to destination
    """
    dest = (dest + "/" + os.path.basename(src)).rstrip("/")
    if pkg_resources.resource_isdir("mambo", src):
        if not os.path.isdir(dest):
            os.makedirs(dest)
        for res in pkg_resources.resource_listdir(__name__, src):
            copy_resource(src + "/" + res, dest)
    else:
        if not os.path.isfile(dest) \
                and os.path.splitext(src)[1] not in [".pyc"]:
            with open(dest, "wb") as f:
                f.write(pkg_resources.resource_string(__name__, src))
        else:
            print("File exists: %s " % dest)


def create_project(project_name, no_assets=True, no_installed_apps=True):
    """
    Create the project
    """
    project_dir = get_project_dir_path(project_name)
    init_tpl = pkg_resources.resource_string(__name__, '%s/init.py' % (SKELETON_DIR))
    app_tpl = pkg_resources.resource_string(__name__, '%s/serve.py' % (SKELETON_DIR))
    propel_tpl = pkg_resources.resource_string(__name__, '%s/propel.yml' % (SKELETON_DIR))
    config_tpl = pkg_resources.resource_string(__name__, '%s/config.py' % (SKELETON_DIR))
    config_installed_apps_tpl = pkg_resources.resource_string(__name__, '%s/config_installed_apps.tpl' % (SKELETON_DIR))
    config_project_tpl = pkg_resources.resource_string(__name__, '%s/config_project.py' % (SKELETON_DIR))
    model_tpl = pkg_resources.resource_string(__name__, '%s/models.py' % (SKELETON_DIR))
    gitignore_tpl = pkg_resources.resource_string(__name__, '%s/gitignore' % (SKELETON_DIR))
    views_tpl = pkg_resources.resource_string(__name__, '%s/views.py' % (SKELETON_DIR))
    babel_tpl = pkg_resources.resource_string(__name__, '%s/babel.cfg' % (SKELETON_DIR))

    if not no_installed_apps:
        config_tpl = config_tpl.replace("##INSTALLED_APPS##", config_installed_apps_tpl)

    serve_file = "%s/serve.py" % CWD
    requirements_txt = "%s/requirements.txt" % CWD
    propel_yml = "%s/propel.yml" % CWD
    config_py = "%s/config.py" % APPLICATION_DIR
    model_py = "%s/models.py" % APPLICATION_DIR
    babel_cfg = "%s/babel.cfg" % APPLICATION_DIR
    gitignore = "%s/.gitignore" % CWD
    views = "%s/views.py" % project_dir
    config_project_py = "%s/config.py" % project_dir
    init_py = "%s/__init__.py" % APPLICATION_DIR

    dirs = [
        APPLICATION_DIR,
        APPLICATION_DATA_DIR,
        project_dir
    ]
    for d in dirs:
        if not os.path.isdir(d):
            os.makedirs(d)

    files = [
        ("%s/__init__.py" % project_dir, "# Mambo"),
        (init_py, init_tpl),
        (config_py, config_tpl),
        (model_py, model_tpl),
        (serve_file, app_tpl.format(project_name=project_name)),
        (requirements_txt, "%s~=%s" % (__title__, __version__)),
        (propel_yml, propel_tpl.format(project_name=project_name)),
        (gitignore, gitignore_tpl),
        (views, views_tpl),
        (config_project_py, config_project_tpl),
        (babel_cfg, babel_tpl)
    ]
    for f in files:
        if not os.path.isfile(f[0]):
            with open(f[0], "wb") as f0:
                f0.write(f[1])

    if not no_assets:
        copy_resource("%s/assets/" % SKELETON_DIR, project_dir)

    copy_resource("%s/_data/" % SKELETON_DIR, APPLICATION_DATA_DIR)

# ------------------------------------------------------------------------------
serve = None


def get_propel_config(cwd, key):
    with open("%s/%s" % (cwd, "propel.yml")) as f:
        config = yaml.load(f)
    return config.get(key)


def get_push_remotes_list(cwd, key=None):
    """
    Returns the remote hosts in propel
    :param cwd:
    :param key:
    :param file:
    :return: list
    """
    config = get_propel_config(cwd, "deploy-remotes")
    return config[key] if key else [v for k, l in config.items() for v in l]


def git_push_to_master(cwd, hosts, name="all", force=False):
    """
    To push to master
    :param cwd:
    :param hosts:
    :param force:
    :return:
    """

    def process_output(line):
        print(line)

    with sh.pushd(cwd):
        name = "mambo_deploy_%s" % name

        if sh.git("status", "--porcelain").strip():
            raise Exception("Repository is UNCLEAN. Commit your changes")

        remote_list = sh.git("remote").strip().split()
        if name in remote_list:
            sh.git("remote", "remove", name)
        sh.git("remote", "add", name, hosts[0])

        if len(hosts) > 1:
            for h in hosts:
                sh.git("remote", "set-url", name, "--push", "--add", h)

        _o = ["push", name, "master"]
        if force:
            _o.append("--force")
        sh.git(*_o, _out=process_output)
        sh.git("remote", "remove", name)


# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------
def catch_exception(func):
    @functools.wraps(func)
    def decorated_view(*args, **kwargs):
        try:
            func(*args, **kwargs)
        except Exception as ex:
            print("-" * 80)
            print("Exception Uncaught")
            print("-" * 80)
            print(ex)
            print("-" * 80)
    return decorated_view


def cwd_to_sys_path():
    sys.path.append(CWD)


def project_name(name):
    return re.compile('[^a-zA-Z0-9_]').sub("", name)


def header(title=None):
    print(__doc__)
    print("v. %s" % __version__)
    print("_" * 80)
    if title:
        print("** %s **" % title)


def build_assets(app):
    from webassets.script import CommandLineEnvironment
    assets_env = app.jinja_env.assets_environment
    log = logging.getLogger('webassets')
    log.addHandler(logging.StreamHandler())
    log.setLevel(logging.DEBUG)
    cmdenv = CommandLineEnvironment(assets_env, log)
    cmdenv.build()


@click.group()
def cli(): pass


@cli.command("create")
@click.argument("app_name")
@click.option("--no-assets", "-x", is_flag=True)
@click.option("--no-installed-apps", "-z", is_flag=True)
@catch_exception
def create(app_name, no_assets, no_installed_apps):
    """  Create a new Application in the current directory """

    app = project_name(app_name)

    header("Creating new app ...")
    print("")
    print("- Name: %s " % app)

    create_project(app, no_assets, no_installed_apps)

    print("")
    print("----- That was so Mambo! ----")
    print("")
    print("- Your new Mambo app [ %s ] has been created" % app)
    print("- Location: [ application/%s ]" % app)
    print("")
    print("> What's next?")
    print("- Edit the config [ application/config.py ] ")
    print("- If necessary edit and run the command [ mambo setup ]")
    print("- Launch app on development mode, run [ app='%s' mambo serve ]" % app)
    print("")
    print("*" * 80)


@cli.command("serve")
@click.option("--port", "-p", default=5000)
@catch_exception
def server(port):
    """ Serve application in development mode """

    header("Serving application in development mode ... ")
    print("")
    print("- Port: %s" % port)
    print("")
    cwd_to_sys_path()
    serve.app.run(debug=True, host='0.0.0.0', port=port)

@cli.command("syncdb")
def syncdb():
    """ Sync database Create new tables etc... """

    print("Syncing up database...")
    cwd_to_sys_path()
    serve = import_string("serve")
    if serve.app.db and serve.app.db.Model:
        serve.app.db.create_all()
        for m in serve.app.db.Model.__subclasses__():
            if hasattr(m, "_syncdb"):
                print("Sync up model: %s ..." % m.__name__)
                getattr(m, "_syncdb")()

    print("Done")


@cli.command("assets2s3")
@catch_exception
def assets2s3():
    """ Upload assets files to S3 """
    import flask_s3

    header("Assets2S3...")
    print("")
    print("Building assets files..." )
    print("")
    build_assets(serve.app)
    print("")
    print("Uploading assets files to S3 ...")
    flask_s3.create_all(serve.app)
    print("")


@cli.command("deploy")
@click.option("--remote", "-r", default=None)
@click.option("--force", "-f", is_flag=True, default=False)
@catch_exception
def deploy(remote, force):
    """ To deploy application to a git repository """

    remote_name = remote or "ALL"
    print("Deploying application to remote: %s " % remote_name)
    print("...")
    hosts = get_push_remotes_list(CWD, remote or None)
    git_push_to_master(cwd=CWD, hosts=hosts, name=remote_name, force=force)
    print("Done!")
    
@cli.command()
def version():
    print("-" * 80)
    print(__version__)
    print("-" * 80)

# @cli.command("babel-build")
# def babelbuild():
#     print("Babel Build....")
#     #sh.pybabel()

# ------------------------------------------------------------------------------
# ------------------------------------------------------------------------------

def cmd():
    """
    Help to run the command line
    :return:
    """
    global serve

    if os.path.isfile(os.path.join(os.path.join(CWD, "serve.py"))):
        cwd_to_sys_path()
        serve = import_string("serve")
    else:
        print("-" * 80)
        print("** Missing << 'serve.py' >> @ %s" % CWD)
        print("-" * 80)

    [cmd(cli.command, click) for cmd in MamboCLI.__subclasses__()]
    cli()




import datetime
import flask_login
from flask_login import current_user
import flask_login
from flask import current_app
from mambo import (register_package, get_config, abort, send_mail, url_for, views, 
                   models, utils, request, redirect, flash, session, emit_signal)
from mambo.exceptions import AppError
from mambo.core import get_installed_app_options
import signals
import exceptions

register_package(__package__)

__version__ = "1.0.0"

ROLES_SUPERADMIN = ["SUPERADMIN"]
ROLES_ADMIN = ROLES_SUPERADMIN + ["ADMIN"]
ROLES_MANAGER = ROLES_ADMIN + ["MANAGER"]
ROLES_CONTRIBUTOR = ROLES_MANAGER + ["EDITOR", "CONTRIBUTOR"]
ROLES_MODERATOR = ROLES_CONTRIBUTOR + ["MODERATOR"]


def main(app, **kwargs):

    @app.before_request
    def force_password_change():
        _ = _get_app_options().get("require_password_change_exclude_endpoints")
        _ = [] if not isinstance(_, list) else _

        exclude_endpoints = ["static", "ContactPage:index", "Index:index",
                             "AuthLogin:logout"] + _

        if current_user and current_user.is_authenticated:
            if request.endpoint \
                    and request.endpoint not in exclude_endpoints:
                if request.endpoint != "AuthAccount:change_password" \
                        and session_get_require_password_change():
                    flash("Password Change is required", "info")
                    return redirect(views.AuthAccount.change_password)

_app_options = None


def _get_app_options():
    global _app_options
    if not _app_options:
        _app_options = get_installed_app_options(__name__)
    return _app_options


def is_authenticated():
    """ A shortcut to check if a user is authenticated """
    return current_user.is_authenticated


def not_authenticated():
    """ A shortcut to check if user not authenticated."""
    return not is_authenticated()


def get_user(id):
    """
    To get a user by id
    :param id:
    :return: AuthUser object
    """
    return models.AuthUser.get(id)


def get_random_password(length=8):
    return utils.generate_random_string(length)


# ------------------------------------------------------------------------------
#
def visible_to(*roles):
    """
    This is a @nav_menu specific function to set the visibility of menu based on
    roles
    :param roles:
    :return: callback fn
    """
    if is_authenticated():
        return True if current_user.has_any_roles(*roles) else False
    return False


# Alias to visible_to
visible_to_superadmins = lambda: visible_to(*ROLES_SUPERADMIN)
visible_to_admins = lambda: visible_to(*ROLES_ADMIN)
visible_to_managers = lambda: visible_to(*ROLES_MANAGER)
visible_to_contributors = lambda: visible_to(*ROLES_CONTRIBUTOR)
visible_to_moderators = lambda: visible_to(*ROLES_MODERATOR)
visible_to_authenticated = lambda: is_authenticated()
visible_to_notauthenticated = lambda: not_authenticated()

# ------------------------------------------------------------------------------
# SIGNUP + LOGIN
def session_login(user_login):
    """
    Login the user to the session
    :param user_login: AuthUserLogin
    :return:
    """
    user_login.user.update(last_login=datetime.datetime.now())
    flask_login.login_user(user_login.user)


def login_with_email(email, password, verify_email=False):
    """
    To login a user with email
    :param email:
    :param password:
    :return:
    """
    def cb():
        userl = models.AuthUserLogin.get_by_email(email)
        if userl and userl.password_hash \
                and userl.password_matched(password):
            if verify_email and not userl.email_verified:
                raise exceptions.VerifyEmailError()
            session_login(userl)
            return userl
        return None
    return signals.on_login(cb)


def signup_with_email(email, password, name, role=None):
    """
    To create a new email login
    :param email:
    :param password:
    :param name:
    :return: AuthUserLogin
    """
    def cb():
        return models\
            .AuthUserLogin.new(login_type=models.AuthUserLogin.TYPE_EMAIL,
                               email=email,
                               password=password.strip(),
                               user_info={
                                    "name": name,
                                    "contact_email": email,
                                    "role": role
                               })
    return signals.on_signup(cb)

# ------------------------------------------------------------------------------
# EMAIL SENDING

def send_mail_password_reset(user_login):
    """
    To reset a user password and send email
    :param user_login: UserLogin object
    :return:
    """
    if user_login.login_type != models.AuthUserLogin.TYPE_EMAIL:
        raise AppError("Invalid login type. Must be the type of email to be sent email to")

    options = _get_app_options()
    delivery = options.get("reset_password_method") or "token"
    token_ttl = get_config("reset_password_token_ttl") or 60
    email_template = options.get("reset_password_email_template") or "reset-password.txt"
    new_password = None

    if delivery.lower() == "token":
        token = user_login.set_temp_login(token_ttl)
        url = url_for(views.AuthLogin.reset_password, token=token, _external=True)
    else:
        new_password = user_login.change_password(random=True)
        url = url_for(views.AuthLogin.index, _external=True)

    send_mail(template=email_template,
              method_=delivery.lower(),
              to=user_login.email,
              name=user_login.user.name,
              email=user_login.email,
              url=url,
              new_password=new_password)


def _create_user_login_verify_email(user_login):
    if user_login.login_type != models.AuthUserLogin.TYPE_EMAIL:
        raise AppError("Invalid login type. Must be the type of email to be sent email to")

    options = _get_app_options()
    token_ttl = options.get("verify_email_token_ttl") or (60 * 24)
    token = user_login.set_email_verified_token(token_ttl)
    url = url_for(views.AuthLogin.verify_email, token=token, _external=True)
    return token, url


def send_mail_verification_email(user_login):
    if user_login.login_type != models.AuthUserLogin.TYPE_EMAIL:
        raise AppError("Invalid login type. Must be the type of email to be sent email to")

    options = _get_app_options()
    email_template = options.get("verify_email_template") or "verify-email.txt"
    token, url = _create_user_login_verify_email(user_login)

    send_mail(template=email_template,
              to=user_login.email,
              name=user_login.user.name,
              email=user_login.email,
              verify_url=url)


def send_mail_signup_welcome(user_login):
    if user_login.login_type != models.AuthUserLogin.TYPE_EMAIL:
        raise AppError(
            "Invalid login type. Must be the type of email to be sent email to")

    options = _get_app_options()
    verify_email = options.get("verify_email") or False
    email_template = options.get("verify_signup_email_template") or "verify-signup-email.txt"
    token, url = _create_user_login_verify_email(user_login)

    send_mail(template=email_template,
              to=user_login.email,
              name=user_login.user.name,
              email=user_login.email,
              verify_url=url,
              verify_email=verify_email)


def session_set_require_password_change(change=True):
    session["auth:require_password_change"] = change

def session_get_require_password_change():
    return session.get("auth:require_password_change")


# ------------------------------------------------------------------------------
# CLI
from mambo.cli import MamboCLI

class CLI(MamboCLI):
    def __init__(self, command, click):
        @command("auth:create-super-admin")
        @click.argument("email")
        def create_super_admin(email):
            """
            To create a super admin by providing the email address
            """
            print("-" * 80)
            print("Mambo Auth: Create Super Admin")
            print("Email: %s" % email)
            try:
                password = get_random_password()
                nl = create_new_login(email=email,
                                      password=password,
                                      name="SuperAdmin",
                                      role=models.AuthRole.SUPERADMIN)
                nl.update(require_password_change=True)
                print("Password: %s" % password)
            except Exception as e:
                print("ERROR: %s" % e)

            print("Done!")

        @command("auth:reset-password")
        @click.argument("email")
        def reset_password(email):
            """
            To reset password by email
            """
            print("-" * 80)
            print("Mambo Auth: Reset Password")
            try:
                ul = models.AuthUserLogin.get_by_email(email)

                if not ul:
                    raise Exception("Email '%s' doesn't exist" % email)
                print(ul.email)
                password = get_random_password()
                ul.change_password(password)
                ul.update(require_password_change=True)
                print("Email: %s" % email)
                print("New Password: %s" % password)
            except Exception as e:
                print("ERROR: %s" % e)

            print("Done!")

        @command("auth:user-info")
        @click.option("--email")
        @click.option("--id")
        def reset_password(email=None, id=None):
            """
            Get the user info by email or ID
            """
            print("-" * 80)
            print("Mambo Auth: User Info")
            print("")
            try:
                if email:
                    ul = models.AuthUserLogin.get_by_email(email)
                    if not ul:
                        raise Exception("Invalid Email address")
                    user_info = ul.user
                elif id:
                    user_info = models.AuthUser.get(id)
                    if not user_info:
                        raise Exception("Invalid User ID")

                k = [
                    ("ID", "id"), ("Name", "name"), ("First Name", "first_name"),
                    ("Last Name", "last_name"), ("Signup", "created_at"),
                    ("Last Login", "last_login"), ("Signup Method", "signup_method"),
                    ("Is Active", "is_active")
                ]
                print("Email: %s" % user_info.get_email_login().email)
                for _ in k:
                    print("%s : %s" % (_[0], getattr(user_info, _[1])))

            except Exception as e:
                print("ERROR: %s" % e)

            print("")
            print("Done!")


# ---

from .decorators import *



"""
Some common functions
"""
from __future__ import division
import inspect
import os
import re
import string
import random
import socket
import subprocess
import functools
import multiprocessing
import threading
from passlib.hash import bcrypt as crypt_engine
from six.moves.urllib.parse import urlparse, urlencode, unquote_plus as urllib_unquote_plus
import datetime
import humanize
import pkg_resources
import urllib
import hashlib
import json
import uuid
from slugify import slugify
from werkzeug.utils import import_string
from distutils.dir_util import (copy_tree as copy_dir,
                                remove_tree as remove_dir,
                                mkpath as make_dirs)
from distutils.file_util import copy_file, move_file
from inflection import (dasherize, underscore, camelize, pluralize, singularize,
                        titleize)


__all__ = [
    "is_email_valid",
    "is_password_valid",
    "is_url_valid",
    "urlencode",
    "urldecode",
    "urlparse",
    "md5",
    "slugify",
    "underscore",
    "dasherize",
    "camelize",
    "pluralize",
    "singularize",
    "titleize",
    "chunk_list",
    "guid",
    "in_any_list"
]

def is_email_valid(email):
    """
    Check if email is valid
    """
    pattern = re.compile(r'[\w\.-]+@[\w\.-]+[.]\w+')
    return bool(pattern.match(email))


def is_password_valid(password):
    """
    Check if a password is valid
    """
    pattern = re.compile(r"^.{4,75}$")
    return bool(pattern.match(password))


def is_url_valid(url):
    """
    Check if url is valid
    """
    pattern = re.compile(
            r'^(?:http|ftp)s?://' # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|' #domain...
            #r'localhost|' #localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})' # ...or ip
            r'(?::\d+)?' # optional port
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)
    return bool(pattern.match(url))


def urldecode(s):
    return urllib_unquote_plus(s)


def md5(value):
    """
    Create MD5
    :param value:
    :return:
    """
    m = hashlib.md5()
    m.update(value)
    return str(m.hexdigest())


def guid():
    """
    Creates and returns a UUID 4 hex value
    :return: string
    """
    return uuid.uuid4().hex


def chunk_list(items, size):
    """
    Return a list of chunks
    :param items: List
    :param size: int The number of items per chunk
    :return: List
    """
    size = max(1, size)
    return [items[i:i + size] for i in range(0, len(items), size)]


def in_any_list(items1, items2):
    """
    Check if any items are in list 3
    :param items1: list
    :param items2: list
    :return:
    """
    return any(i in items2 for i in items1)


def encrypt_string(string):
    """
    Encrypt a string
    """
    return crypt_engine.encrypt(string)


def verify_encrypted_string(string, encrypted_string):
    """
    Verify an encrypted string
    """
    return crypt_engine.verify(string, encrypted_string)


def generate_random_string(length=8):
    """
    Generate a random string
    """
    char_set = string.ascii_uppercase + string.digits
    return ''.join(random.sample(char_set * (length - 1), length))


def generate_random_hash(size=32):
    """
    Return a random hash key
    :param size: The max size of the hash
    :return: string
    """
    return os.urandom(size//2).encode('hex')


def how_old(dob):
    """
    Calculate the age
    :param dob: datetime object
    :return: int
    """
    today = datetime.date.today()
    return today.year - dob.year - ((today.month, today.day) < (dob.month, dob.day))


def is_port_open(port, host="127.0.0.1"):
    """
    Check if a port is open
    :param port:
    :param host:
    :return bool:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((host, int(port)))
        s.shutdown(2)
        return True
    except Exception as e:
        return False

def create_uri(host, user=None, password=None, port=None, path=None, scheme="http"):
    port = ":%s" % port if port else ""
    path = "/%s" % path if path else ""
    user_info = ""
    if user or password:
        user_info = "%s:%s@" % (user or "", password or "")
    return "{scheme}://{user_info}{host}{port}{path}".format(
        scheme=scheme, host=host, user_info=user_info,port=port, path=path)

# ----
# COULD BE DEPRECATED

def get_pkg_resources_filename(pkg, filename=""):
    """
    Returns a string filepath from a package name/path
    It helps get the full path from a package directory
    :param pkg: str - The dotted path of package dir: portofolio.component, or mambo.component.views
    :param filename: str - The file or dir name to use use
    returns str
    """
    return pkg_resources.resource_filename(pkg, filename)


def get_domain_name(url):
    """
    Get the domain name
    :param url:
    :return:
    """
    if not url.startswith("http"):
        url = "http://" + url
    if not is_url_valid(url):
        raise ValueError("Invalid URL '%s'" % url)
    parse = urlparse(url)
    return parse.netloc


def seconds_to_time(sec):
    """
    Convert seconds into time H:M:S
    """
    return "%02d:%02d" % divmod(sec, 60)


def time_to_seconds(t):
    """
    Convert time H:M:S to seconds
    """
    l = list(map(int, t.split(':')))
    return sum(n * sec for n, sec in zip(l[::-1], (1, 60, 3600)))


def convert_bytes(bytes):
    """
    Convert bytes into human readable
    """
    bytes = float(bytes)
    if bytes >= 1099511627776:
        terabytes = bytes / 1099511627776
        size = '%.2fT' % terabytes
    elif bytes >= 1073741824:
        gigabytes = bytes / 1073741824
        size = '%.2fG' % gigabytes
    elif bytes >= 1048576:
        megabytes = bytes / 1048576
        size = '%.2fM' % megabytes
    elif bytes >= 1024:
        kilobytes = bytes / 1024
        size = '%.2fK' % kilobytes
    else:
        size = '%.2fb' % bytes
    return size


def time_ago(dt):
    """
    Return the current time ago
    """
    now = datetime.datetime.now()
    return humanize.naturaltime(now - dt)

# ------------------------------------------------------------------------------
# Background multi processing and threading
# Use the decorators below for background processing

def bg_process(func):
    """
    A multiprocess decorator
    :param func:
    :return:
    """
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        p = multiprocessing.Process(target=func, args=args, kwargs=kwargs)
        p.start()
    return wrapper

def bg_thread(func):
    """
    A threading decorator
    :param func:
    :return:
    """
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        p = threading.Thread(target=func, args=args, kwargs=kwargs)
        p.start()
    return wrapper


# ------------------------------------------------------------------------------

class DotDict(dict):
    """
    Enable use of object notation for dicts.
    The __missing__ method allows chaining creation of nested DotDicts:
    a = DotDict() ; a.foo.bar = 'content'
    Limitation: Expressing keys as attributes limits keys to valid Python
    identifiers.
    """
    __getattr__ = dict.__getitem__
    __delattr__ = dict.__delitem__

    def __setattr__(self, key, value):
        if hasattr(value, 'keys'):
            value = DotDict(value)
        self[key] = value

    def __missing__(self, key):
        self[key] = DotDict()
        return self[key]

    def get(self, keys, default=None):
        """
        To get a value by dot notation
        :param keys: dot notaion string
        :param default:
        :return:
        """
        try:
            d = self
            for k in keys.split("."):
                if not k in d:
                    d[k] = {}
                d = d[k]
            if isinstance(d, bool):
                return d
            return d or default
        except (TypeError, KeyError) as e:
            return default


# ---


class InspectDecoratorCompatibilityError(Exception):
    pass


class _InspectMethodsDecorators(object):
    """
    This class attempt to retrieve all the decorators in a method
    """
    def __init__(self, method):
        self.method = method
        self.decos = []

    def parse(self):
        """
        Return the list of string of all the decorators found
        """
        self._parse(self.method)
        return list(set([deco for deco in self.decos if deco]))

    @classmethod
    def extract_deco(cls, line):
        line = line.strip()
        if line.startswith("@"):
            if "(" in line:
                line = line.split('(')[0].strip()
            return line.strip("@")

    def _parse(self, method):
        argspec = inspect.getargspec(method)
        args = argspec[0]
        if args and args[0] == 'self':
            return argspec
        if hasattr(method, '__func__'):
            method = method.__func__
        if not hasattr(method, '__closure__') or method.__closure__ is None:
            raise InspectDecoratorCompatibilityError

        closure = method.__closure__
        for cell in closure:
            inner_method = cell.cell_contents
            if inner_method is method:
                continue
            if not inspect.isfunction(inner_method) \
                and not inspect.ismethod(inner_method):
                continue
            src = inspect.getsourcelines(inner_method)[0]
            self.decos += [self.extract_deco(line) for line in src]
            self._parse(inner_method)


def get_decorators_list(method):
    """
    Shortcut to InspectMethodsDecorators
    :param method: object
    :return: List
    """
    kls = _InspectMethodsDecorators(method)
    return kls.parse()








from mambo import Mambo, get_config, register_package, abort

register_package(__package__)

__version__ = "1.0.0"

def main(**kwargs):
    """
    Create the Maintenance view
    Must be instantiated

    import maintenance_view
    m = maintenance_view()

    :param template_: The directory containing the view pages
    :return:
    """

    template = "MaintenancePage/index.html"

    class MaintenancePage(Mambo):

        @classmethod
        def _register(cls, app, **kw):

            super(cls, cls)._register(app, **kw)

            @app.before_request
            def on_maintenance():
                return cls.render(_layout=template), 503

    return MaintenancePage



"""
Mambo

Error

"""


class MamboError(Exception):
    """
    This exception is not reserved, but it used for all Mambo exception.
    It helps catch Core problems.
    """
    pass


class AppError(MamboError):
    """
    Use this exception in your application level.
    """
    pass


class ModelError(AppError):
    """
    Use this exception in your model level.
    """
    pass





